/*
 * cancan.h
 * various constants etc
 */

typedef struct 
{               		/* for additional connected sessions */
  int active;			/* display remote output? */
  char *id;			/* session id */
} consess;

/* generic linked list node (never actually created) */
typedef struct defnode {
    struct defnode *next;
    char *sortfield;
} defnode;

/* linked list nodes: keep the next pointer first, then string to sort by */
typedef struct aliasnode {
    struct aliasnode *next;
    char *name;
    char *subst;
} aliasnode;

typedef struct actionnode {
    struct actionnode *next;
    char *pattern;
    char *commands, *label;
    int active;
} actionnode;

typedef struct marknode {
    struct marknode *next;
    char *pattern;
    int attrcode;
} marknode;

typedef struct keynode {
    struct keynode *next;	/* next key definition in list */
    char *name;			/* key name */
    char *sequence;		/* escape sequence sent by terminal */
    void (*funct)();		/* function called when key pressed */
    char *call_data;		/* data passed to function */
} keynode;

/* shared functions: */

/* from cancan.c */
void setsigs();
void connect_sess();
void disconnect_sess();
void exitcancan();
void syserr();
char *skipspace();
void suspend_cancan();
int is_main_sess();
int main_sess();
int find_sess();
void send_to_main_host();
void save_settings();
void check_prompt();
void strdel();
void strins();
void parse_input();
void to_input_line();
void cancel_edit();
int subst_param();
void printver();
void snoop_sess();
void flashparen();
void put_in_history();
void record_line();

/* from mapping.c */
void remove_steps();
void retrace_steps();
void show_automap();
void add_dir_to_map();

/* from tel.c */
int doconnect();
int read_socket();
void send_term_size();
void send_to_host();
void flush_output_buffer();

/* from list.c */
void add_aliasnode();
void add_actionnode();
void add_marknode();
void add_keynode();
aliasnode **lookup_alias();
actionnode **lookup_action();
actionnode **lookup_action_pattern();
marknode **lookup_marker();
keynode **lookup_key();
void delete_aliasnode();
void delete_actionnode();
void delete_marknode();
void delete_keynode();

/* from beam.c */
void message_edit();
int process_message();
void childdeathsig();

#define MAXFDSCAN 32		/* max number of fd to scan in select() */
#define MAXHIST 128		/* number of history lines kept */
#define DELIM " ;:.?!,"		/* word delimiting characters */
#define CAPLEN 20		/* max length of a terminal capability */
#define BUFSIZE 2048		/* general buffer size */
#define LINELEN 1024		/* maximum line length */
#define OUTBUFSIZE 2048		/* output buffer size */
#define PARAMLEN 256		/* maximum length of &0..&9 params */
#define MAXMAPLEN 500		/* maximum length of automapped path */
#define WORDSKEPT 512		/* number of words kept for TAB-completion */
#define MPI "~$#E"		/* MUME protocol introducer */
#define SOCKBUFSIZE 4096	/* socket buffer size for read */
#define MAXSUBOPT 256		/* max length of suboption string */
#define MAXARGS 16		/* max number of arguments to editor */
#define FLASHDELAY 500000	/* time of parentheses flash in microsecs */
#define CMDSEP ';'		/* command separator character */
#define KBD_TIMEOUT 35000	/* timeout for keyboard read in microsecs;
				   35000 should be enough for 300+ baud */

#define LM_NOECHO 1	/* no local echo */
#define LM_CHAR 2	/* char-by-char mode (no line editing) */

/* editing session control */
typedef struct editsess_ 
{
  int key;		/* session identifier (-1 if cancelled) */
  int pid;		/* pid of child */
  char *descr;  	/* short description of what we are editing */
  char *file;		/* name of temporary file */
  time_t ctime; 	/* time when temp file was created (upper bound) */
  long oldsize; 	/* original file size */
  struct editsess_ *next;
} editsess;

/* shared vars */
/* from cancan.c */
extern int linemode;		/* line mode with flags as above (LM_*) */
extern int sockfd;		/* socket file descriptor */

extern int cols, lines;		/* terminal window size */
extern int line0, col0;		/* origin of input line */
extern char promptstr[];	/* prompt string */
extern char edbuf[];		/* input line buffer */
extern int edlen;		/* characters in edbuf */
extern int pos;			/* cursor position in edbuf */
extern char edattrbeg[], edattrend[];

extern aliasnode *aliases;
extern actionnode *actions;
extern marknode *markers;
extern keynode *keydefs;

extern char initstr[];
extern int speedwalk;

extern int new_settings;	/* set this if we need to save def file */
extern int data_sent;		/* incremented each time we send_to_host */
extern int verbose;		/* this is set if cmd is user-typed */

extern FILE *capfile;
extern char prefixstr[];
extern char inserted_next[];
extern int verbatim;

extern int wrap;

extern char *hist[];
extern int curline;
extern int pickline;

/* from beam.c */
extern char edit_start[];
extern char edit_end[];
extern editsess *edit_sess;

#define CONTROL(c) ((c) & 0x1f)

/* macros to find cursor position from input buffer position */
#define CURLINE(pos) (((pos) + col0) / (cols - 1) + line0)
#define CURCOL(pos) (((pos) + col0) % (cols - 1))

/* macros to match parentheses */
#define ISRPAREN(c) ((c) == ')' || (c) == ']' || (c) == '}')
#define ISLPAREN(c) ((c) == '(' || (c) == '[' || (c) == '{')
#define LPAREN(c) ((c) == ')' ? '(' : ((c) == ']' ? '[' : '{'))

#define MIN(a, b) ((a) < (b) ? (a) : (b))
#define MAX(a, b) ((a) > (b) ? (a) : (b))

/*
 * Attribute codes: bit 0-3 for foreground colour or effects,
 * 4-7 for background colour. Colour #8 is "none", so 0x88 is "no attribute".
 */
#define COLOURS 8		/* number of ANSI colours */
#define NOATTRIB COLOURS	/* no colour change, use default */
#define BOLD (COLOURS + 1)
#define UNDERLINE (COLOURS + 2)
#define INVERSE (COLOURS + 3)
#define TOK_ON (COLOURS + 4)	/* these two used by attr_token() */
#define TOK_INVALID (-1)

/* constructors / accessors for attribute codes */
#define ATTRCODE(fg, bg) ((fg) | ((bg << 4)))
#define FOREGROUND(attrcode) ((attrcode) & 0xf)
#define BACKGROUND(attrcode) ((attrcode) >> 4)

#define NOATTRCODE ATTRCODE(NOATTRIB, NOATTRIB)

/*
 * NCSA telnet 2.2 doesn't reset the colour when it receives "esc [ m",
 * so we must know what the normal colours are in order to reset it.
 * These colours can be changed with the #stdcolour command.
 */
#ifdef TELNETBUG
#define DEFAULTFG 7		/* make default white text */
#define DEFAULTBG 4		/* on blue background */
#endif
