/*
 * edit.c
 * line editing functions for cancan
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include "cancan.h"
#include "edit.h"
#include "term.h"

/*
 * insert a typed character (if it is printable)
 */
void insert_char(c)
int c;
{
    char ch;
    if(edlen < LINELEN - 2 && c >= ' ') {
	ch = c;
	insertchars(&ch, 1);
	if(ISRPAREN(c)) flashparen(c);
    }
}

/*
 * redraw the input line
 */
void redraw_input_line()
{
    clearinpline();
    printf("\n");
    redraw_input();
}

/*
 * transpose two characters to the left
 */
void transpose_chars()
{
    int i, cl, cc;
    unsigned char c;
    if(pos > 1) {
	cl = CURLINE(pos); cc = CURCOL(pos);
	if(pos < edlen)
	    pos++;
	i = pos - 2;
	c = edbuf[i + 1]; edbuf[i + 1] = edbuf[i]; edbuf[i] = c;
	optmove(cl, cc, CURLINE(i), CURCOL(i));
	putchar(edbuf[i]);
	optmove(CURLINE(i), CURCOL(i) + 1, CURLINE(i + 1), CURCOL(i + 1));
	putchar(edbuf[i + 1]);
	optmove(CURLINE(i + 1), CURCOL(i + 1) + 1, CURLINE(pos),
		CURCOL(pos));
	fflush(stdout);
    }
}

/*
 * erase everything to the end of line
 */
void kill_to_eol()
{
    printf("%s", clreoln);
    if(CURLINE(edlen) > CURLINE(pos)) {
	printf("\n%s", clreoscr);
	optmove(CURLINE(pos) + 1, 0, CURLINE(pos), CURCOL(pos));
    }
    edbuf[pos] = '\0';
    edlen = pos;
    fflush(stdout);
}

/*
 * move cursor to end of line
 */
void end_of_line()
{
    optmove(CURLINE(pos), CURCOL(pos), CURLINE(edlen), CURCOL(edlen));
    fflush(stdout);
    pos = edlen;
}

/*
 * move cursor to beginning of line
 */
void beginning_of_line()
{
    optmove(CURLINE(pos), CURCOL(pos), line0, col0);
    pos = 0;
    fflush(stdout);
}

/*
 * delete a character to the right
 */
void del_char_right()
{
    if(pos < edlen)
	deletechars(1);
}

/*
 * delete a character to the left
 */
void del_char_left()
{
    if(pos) {
	pos--;
	optmove(CURLINE(pos + 1), CURCOL(pos + 1), CURLINE(pos), CURCOL(pos));
	deletechars(1);
    }
}

/*
 * enter a line into history, but don't do anything else
 */
void enter_into_history()
{
    clearinpline();
    put_in_history(edbuf);
    pickline = curline;
    strcpy(edbuf, prefixstr); pos = edlen = strlen(edbuf);
}

/*
 * enter a line
 */
void enter_line()
{
    char *p;
    printf("\n");
    if(line0 < lines - 1) line0++;
    if(capfile) fprintf(capfile, "%s\n", edbuf);
#ifdef NEWBIE
    denewbiefy();
#endif
    if(verbatim && edbuf[0] != '#') {
	edbuf[edlen] = '\n';
	send_to_host(edbuf, edlen + 1);
	edbuf[edlen] = '\0';
    } else {
        verbose = 1;
	parse_input(edbuf, 1);
	verbose = 0;
    }
    if(new_settings) {
	save_settings();  /* save new definitions user just defined */
	new_settings = 0;
    }
    check_prompt();
    record_line(edbuf);
    /* don't put identical lines in history, nor short ones */
    p = hist[curline ? curline - 1 : MAXHIST - 1];
    if(!p || (strcmp(edbuf, p) != 0 && edlen > 1))
	put_in_history(edbuf);
    pickline = curline;
    if(inserted_next[0]) {
	strcpy(edbuf, inserted_next);
	inserted_next[0] = '\0';
    } else
	strcpy(edbuf, prefixstr);
    pos = edlen = strlen(edbuf);
    redraw_input();
}

/*
 * move one word forward
 */
void next_word()
{
    int i;
    for(i = pos; !isalnum(edbuf[i]) && edbuf[i]; i++);
    while(isalnum(edbuf[i])) i++;
    optmove(CURLINE(pos), CURCOL(pos), CURLINE(i), CURCOL(i));
    pos = i;
    fflush(stdout);
}

/*
 * move one word backward
 */
void prev_word()
{
    int i;
    for(i = pos; i && !isalnum(edbuf[i - 1]); i--);
    while(i && isalnum(edbuf[i - 1])) i--;
    optmove(CURLINE(pos), CURCOL(pos), CURLINE(i), CURCOL(i));
    pos = i;
    fflush(stdout);
}

/*
 * delete word to the right
 */
void del_word_right()
{
    int i;
    for(i = pos; !isalnum(edbuf[i]) && i < edlen; i++);
    while(isalnum(edbuf[i])) i++;
    deletechars(i - pos);
}

/*
 * delete word to the left
 */
void del_word_left()
{
    int i;
    for(i = pos; i && !isalnum(edbuf[i - 1]); i--);
    while(i && isalnum(edbuf[i - 1])) i--;
    optmove(CURLINE(pos), CURCOL(pos), CURLINE(i), CURCOL(i));
    i = pos - i; pos -= i;
    deletechars(i);
}

/*
 * get previous line from history list
 */
void prev_line()
{
    int i = pickline - 1;
    if(i < 0) i = MAXHIST - 1;
    if(hist[i]) {
	if(hist[pickline]) free(hist[pickline]);
	hist[pickline] = strdup(edbuf);
	pickline = i;
	clearinpline();
	strcpy(edbuf, hist[pickline]);
	pos = edlen = strlen(edbuf);
	redraw_input();
    }
}

/*
 * get next line from history list
 */
void next_line()
{
    int i = pickline + 1;
    if(i == MAXHIST) i = 0;
    if(hist[i]) {
	if(hist[pickline]) free(hist[pickline]);
	hist[pickline] = strdup(edbuf);
	pickline = i;
	clearinpline();
	strcpy(edbuf, hist[pickline]);
	edlen = pos = strlen(edbuf);
	redraw_input();
    }
}

/*
 * move one char backward
 */
void prev_char()
{
    if(pos) {
	pos--;
	optmove(CURLINE(pos + 1), CURCOL(pos + 1), CURLINE(pos), CURCOL(pos));
	fflush(stdout);
    }
}

/*
 * move one char forward
 */
void next_char()
{
    if(pos < edlen) {
	pos++;
	optmove(CURLINE(pos - 1), CURCOL(pos - 1), CURLINE(pos), CURCOL(pos));
	fflush(stdout);
    }
}

/*
 * execute string as if typed
 */
void key_run_command(cmd)
char *cmd;
{
    record_line(cmd);
    clearinpline();
    printf("%s\n", cmd);
    if(capfile) fprintf(capfile, "%s\n", cmd);
    parse_input(cmd, 1);
    check_prompt();
    redraw_input();
}

