/*
 * list utility functions:
 * I've given up the idea of having one generic list type, so everything
 * is duplicated in functions for aliases, actions, markers and keydefs.
 * Better alternatives, anyone?
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include "cancan.h"

char *strdup();

/*
 * generic list node adding routine
 */
void add_node(newnode, base)
defnode *newnode, **base;
{
#ifndef NOSORT
    while((*base) && strcmp((*base)->sortfield, newnode->sortfield) < 0)
	base = &(*base)->next;
#endif
    newnode->next = *base;
    *base = newnode;
}

/*
 * add a node to the alias list (sorted unless NOSORT is defined)
 */
void add_aliasnode(name, subst)
char *name, *subst;
{
    aliasnode *new = (aliasnode*)malloc(sizeof(aliasnode));
    if(!new) syserr("malloc");
    new->name = strdup(name);
    new->subst = strdup(subst);
    if(!new->name || !new->subst) syserr("strdup");
    add_node((defnode*)new, (defnode**)&aliases);
}
    
/*
 * add a node to the action list (sorted unless NOSORT is defined)
 */
void add_actionnode(pattern, commands, label, active)
char *pattern, *commands, *label; int active;
{
    actionnode *new = (actionnode*)malloc(sizeof(actionnode));
    if(!new) syserr("malloc");
    new->pattern = strdup(pattern);
    new->commands = strdup(commands);
    new->label = strdup(label);
    new->active = active;
    if(!new->pattern || !new->commands || !new->label) syserr("strdup");
    add_node((defnode*)new, (defnode**)&actions);
}

/*
 * add a node to the marker list (sorted unless NOSORT is defined)
 */
void add_marknode(pattern, attrcode)
char *pattern; int attrcode;
{
    marknode *new = (marknode*)malloc(sizeof(marknode));
    if(!new) syserr("malloc");
    new->pattern = strdup(pattern);
    new->attrcode = attrcode;
    if(!new->pattern) syserr("strdup");
    add_node((defnode*)new, (defnode**)&markers);
}

/*
 * add a node to the keydef list (sorted unless NOSORT is defined)
 */
void add_keynode(name, sequence, funct, call_data)
char *name, *sequence; void (*funct)(); char *call_data;
{
    keynode *new = (keynode*)malloc(sizeof(keynode));
    if(!new) syserr("malloc");
    new->name = strdup(name);
    new->sequence = strdup(sequence);
    new->funct = funct;
    new->call_data = call_data ? strdup(call_data) : call_data;
    if(!new->name || !new->sequence) syserr("strdup");
    add_node((defnode*)new, (defnode**)&keydefs);
}

/*
 * look up an alias node by name:
 * return pointer to pointer to node or a pointer to NULL if nothing found
 */
aliasnode **lookup_alias(name)
char *name;
{
    aliasnode **p = &aliases;
    while(*p && strcmp(name, (*p)->name) != 0)
	p = &(*p)->next;
    return p;
}

/*
 * look up an action node by label:
 * return pointer to pointer to node or a pointer to NULL if nothing found
 */
actionnode **lookup_action(label)
char *label;
{
    actionnode **p = &actions;
    while(*p && strcmp(label, (*p)->label) != 0)
	p = &(*p)->next;
    return p;
}

/*
 * look up an action node by pattern:
 * return pointer to pointer to node or a pointer to NULL if nothing found
 */
actionnode **lookup_action_pattern(pattern)
char *pattern;
{
    actionnode **p = &actions;
    while(*p && strcmp(pattern, (*p)->pattern) != 0)
	p = &(*p)->next;
    return p;
}

/*
 * look up a marker node by pattern:
 * return pointer to pointer to node or a pointer to NULL if nothing found
 */
marknode **lookup_marker(pattern)
char *pattern;
{
    marknode **p = &markers;
    while(*p && strcmp(pattern, (*p)->pattern) != 0)
	p = &(*p)->next;
    return p;
}

/*
 * look up a keydef node by pattern:
 * return pointer to pointer to node or a pointer to NULL if nothing found
 */
keynode **lookup_key(name)
char *name;
{
    keynode **p = &keydefs;
    while(*p && strcmp(name, (*p)->name) != 0)
	p = &(*p)->next;
    return p;
}

/*
 * delete an alias node, given a pointer to its precessor's pointer
 */
void delete_aliasnode(base)
aliasnode **base;
{
    aliasnode *p = *base;
    free(p->name);
    free(p->subst);
    *base = p->next;
    free((char*)p);
}

/*
 * delete an action node, given a pointer to its precessor's pointer
 */
void delete_actionnode(base)
actionnode **base;
{
    actionnode *p = *base;
    free(p->pattern);
    free(p->commands);
    free(p->label);
    *base = p->next;
    free((char*)p);
}

/*
 * delete an marker node, given a pointer to its precessor's pointer
 */
void delete_marknode(base)
marknode **base;
{
    marknode *p = *base;
    free(p->pattern);
    *base = p->next;
    free((char*)p);
}

/*
 * delete a keydef node, given a pointer to its precessor's pointer
 */
void delete_keynode(base)
keynode **base;
{
    keynode *p = *base;
    free(p->name);
    free(p->sequence);
    if(p->call_data) free(p->call_data);
    *base = p->next;
    free((char*)p);
}
